Package: gcapc
Title: GC Aware Peak Caller
Version: 1.37.0
Author: Mingxiang Teng and Rafael A. Irizarry
Maintainer: Mingxiang Teng <tengmx@gmail.com>
Description: Peak calling for ChIP-seq data with
    consideration of potential GC bias in sequencing
    reads. GC bias is first estimated with generalized
    linear mixture models using effective GC strategy,
    then applied into peak significance estimation.
Depends: R (>= 3.4)
Imports: BiocGenerics, Seqinfo, S4Vectors, IRanges, Biostrings,
        BSgenome, GenomicRanges, Rsamtools, GenomicAlignments,
        matrixStats, MASS, splines, grDevices, graphics, stats, methods
VignetteBuilder: knitr
Suggests: BiocStyle, knitr, rmarkdown, BSgenome.Hsapiens.UCSC.hg19,
        BSgenome.Mmusculus.UCSC.mm10
URL: https://github.com/tengmx/gcapc
License: GPL-3
biocViews: Sequencing, ChIPSeq, BatchEffect, PeakDetection
RoxygenNote: 6.0.1
git_url: https://git.bioconductor.org/packages/gcapc
git_branch: devel
git_last_commit: 16b89e7
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:46:29 UTC; biocbuild
